package com.sun.xml.internal.ws.wsdl.parser;

import javax.xml.stream.Location;
import org.xml.sax.helpers.LocatorImpl;
import org.xml.sax.Locator;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLBoundFault;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLFault;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLOutput;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLInput;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLMessage;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLBoundOperation;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLPortType;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLBoundPortType;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLOperation;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLPort;
import com.sun.xml.internal.ws.streaming.XMLStreamReaderUtil;
import javax.xml.stream.XMLStreamReader;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLService;
import com.sun.xml.internal.ws.api.wsdl.parser.WSDLParserExtensionContext;
import com.sun.xml.internal.ws.api.wsdl.parser.WSDLParserExtension;

final class WSDLParserExtensionFacade extends WSDLParserExtension
{
    private final WSDLParserExtension[] extensions;
    
    WSDLParserExtensionFacade(WSDLParserExtension... extensions) {
        assert extensions != null;
        this.extensions = extensions;
        return;
    }
    
    public void start(WSDLParserExtensionContext context) {
        for (WSDLParserExtension e : this.extensions) {
            e.start(context);
        }
    }
    
    public boolean serviceElements(EditableWSDLService service, XMLStreamReader reader) {
        WSDLParserExtension[] extensions = this.extensions;
        int length = extensions.length;
        int i = 0;
        while (i < length) {
            WSDLParserExtension e = extensions[i];
            if (e.serviceElements(service, reader)) {
                return true;
            }
            ++i;
        }
        XMLStreamReaderUtil.skipElement(reader);
        return true;
    }
    
    public void serviceAttributes(EditableWSDLService service, XMLStreamReader reader) {
        for (WSDLParserExtension e : this.extensions) {
            e.serviceAttributes(service, reader);
        }
    }
    
    public boolean portElements(EditableWSDLPort port, XMLStreamReader reader) {
        WSDLParserExtension[] extensions = this.extensions;
        int length = extensions.length;
        int i = 0;
        while (i < length) {
            WSDLParserExtension e = extensions[i];
            if (e.portElements(port, reader)) {
                return true;
            }
            ++i;
        }
        if (this.isRequiredExtension(reader)) {
            port.addNotUnderstoodExtension(reader.getName(), this.getLocator(reader));
        }
        XMLStreamReaderUtil.skipElement(reader);
        return true;
    }
    
    public boolean portTypeOperationInput(EditableWSDLOperation op, XMLStreamReader reader) {
        for (WSDLParserExtension e : this.extensions) {
            e.portTypeOperationInput(op, reader);
        }
        return false;
    }
    
    public boolean portTypeOperationOutput(EditableWSDLOperation op, XMLStreamReader reader) {
        for (WSDLParserExtension e : this.extensions) {
            e.portTypeOperationOutput(op, reader);
        }
        return false;
    }
    
    public boolean portTypeOperationFault(EditableWSDLOperation op, XMLStreamReader reader) {
        for (WSDLParserExtension e : this.extensions) {
            e.portTypeOperationFault(op, reader);
        }
        return false;
    }
    
    public void portAttributes(EditableWSDLPort port, XMLStreamReader reader) {
        for (WSDLParserExtension e : this.extensions) {
            e.portAttributes(port, reader);
        }
    }
    
    public boolean definitionsElements(XMLStreamReader reader) {
        WSDLParserExtension[] extensions = this.extensions;
        int length = extensions.length;
        int i = 0;
        while (i < length) {
            WSDLParserExtension e = extensions[i];
            if (e.definitionsElements(reader)) {
                return true;
            }
            ++i;
        }
        XMLStreamReaderUtil.skipElement(reader);
        return true;
    }
    
    public boolean bindingElements(EditableWSDLBoundPortType binding, XMLStreamReader reader) {
        WSDLParserExtension[] extensions = this.extensions;
        int length = extensions.length;
        int i = 0;
        while (i < length) {
            WSDLParserExtension e = extensions[i];
            if (e.bindingElements(binding, reader)) {
                return true;
            }
            ++i;
        }
        if (this.isRequiredExtension(reader)) {
            binding.addNotUnderstoodExtension(reader.getName(), this.getLocator(reader));
        }
        XMLStreamReaderUtil.skipElement(reader);
        return true;
    }
    
    public void bindingAttributes(EditableWSDLBoundPortType binding, XMLStreamReader reader) {
        for (WSDLParserExtension e : this.extensions) {
            e.bindingAttributes(binding, reader);
        }
    }
    
    public boolean portTypeElements(EditableWSDLPortType portType, XMLStreamReader reader) {
        WSDLParserExtension[] extensions = this.extensions;
        int length = extensions.length;
        int i = 0;
        while (i < length) {
            WSDLParserExtension e = extensions[i];
            if (e.portTypeElements(portType, reader)) {
                return true;
            }
            ++i;
        }
        XMLStreamReaderUtil.skipElement(reader);
        return true;
    }
    
    public void portTypeAttributes(EditableWSDLPortType portType, XMLStreamReader reader) {
        for (WSDLParserExtension e : this.extensions) {
            e.portTypeAttributes(portType, reader);
        }
    }
    
    public boolean portTypeOperationElements(EditableWSDLOperation operation, XMLStreamReader reader) {
        WSDLParserExtension[] extensions = this.extensions;
        int length = extensions.length;
        int i = 0;
        while (i < length) {
            WSDLParserExtension e = extensions[i];
            if (e.portTypeOperationElements(operation, reader)) {
                return true;
            }
            ++i;
        }
        XMLStreamReaderUtil.skipElement(reader);
        return true;
    }
    
    public void portTypeOperationAttributes(EditableWSDLOperation operation, XMLStreamReader reader) {
        for (WSDLParserExtension e : this.extensions) {
            e.portTypeOperationAttributes(operation, reader);
        }
    }
    
    public boolean bindingOperationElements(EditableWSDLBoundOperation operation, XMLStreamReader reader) {
        WSDLParserExtension[] extensions = this.extensions;
        int length = extensions.length;
        int i = 0;
        while (i < length) {
            WSDLParserExtension e = extensions[i];
            if (e.bindingOperationElements(operation, reader)) {
                return true;
            }
            ++i;
        }
        XMLStreamReaderUtil.skipElement(reader);
        return true;
    }
    
    public void bindingOperationAttributes(EditableWSDLBoundOperation operation, XMLStreamReader reader) {
        for (WSDLParserExtension e : this.extensions) {
            e.bindingOperationAttributes(operation, reader);
        }
    }
    
    public boolean messageElements(EditableWSDLMessage msg, XMLStreamReader reader) {
        WSDLParserExtension[] extensions = this.extensions;
        int length = extensions.length;
        int i = 0;
        while (i < length) {
            WSDLParserExtension e = extensions[i];
            if (e.messageElements(msg, reader)) {
                return true;
            }
            ++i;
        }
        XMLStreamReaderUtil.skipElement(reader);
        return true;
    }
    
    public void messageAttributes(EditableWSDLMessage msg, XMLStreamReader reader) {
        for (WSDLParserExtension e : this.extensions) {
            e.messageAttributes(msg, reader);
        }
    }
    
    public boolean portTypeOperationInputElements(EditableWSDLInput input, XMLStreamReader reader) {
        WSDLParserExtension[] extensions = this.extensions;
        int length = extensions.length;
        int i = 0;
        while (i < length) {
            WSDLParserExtension e = extensions[i];
            if (e.portTypeOperationInputElements(input, reader)) {
                return true;
            }
            ++i;
        }
        XMLStreamReaderUtil.skipElement(reader);
        return true;
    }
    
    public void portTypeOperationInputAttributes(EditableWSDLInput input, XMLStreamReader reader) {
        for (WSDLParserExtension e : this.extensions) {
            e.portTypeOperationInputAttributes(input, reader);
        }
    }
    
    public boolean portTypeOperationOutputElements(EditableWSDLOutput output, XMLStreamReader reader) {
        WSDLParserExtension[] extensions = this.extensions;
        int length = extensions.length;
        int i = 0;
        while (i < length) {
            WSDLParserExtension e = extensions[i];
            if (e.portTypeOperationOutputElements(output, reader)) {
                return true;
            }
            ++i;
        }
        XMLStreamReaderUtil.skipElement(reader);
        return true;
    }
    
    public void portTypeOperationOutputAttributes(EditableWSDLOutput output, XMLStreamReader reader) {
        for (WSDLParserExtension e : this.extensions) {
            e.portTypeOperationOutputAttributes(output, reader);
        }
    }
    
    public boolean portTypeOperationFaultElements(EditableWSDLFault fault, XMLStreamReader reader) {
        WSDLParserExtension[] extensions = this.extensions;
        int length = extensions.length;
        int i = 0;
        while (i < length) {
            WSDLParserExtension e = extensions[i];
            if (e.portTypeOperationFaultElements(fault, reader)) {
                return true;
            }
            ++i;
        }
        XMLStreamReaderUtil.skipElement(reader);
        return true;
    }
    
    public void portTypeOperationFaultAttributes(EditableWSDLFault fault, XMLStreamReader reader) {
        for (WSDLParserExtension e : this.extensions) {
            e.portTypeOperationFaultAttributes(fault, reader);
        }
    }
    
    public boolean bindingOperationInputElements(EditableWSDLBoundOperation operation, XMLStreamReader reader) {
        WSDLParserExtension[] extensions = this.extensions;
        int length = extensions.length;
        int i = 0;
        while (i < length) {
            WSDLParserExtension e = extensions[i];
            if (e.bindingOperationInputElements(operation, reader)) {
                return true;
            }
            ++i;
        }
        XMLStreamReaderUtil.skipElement(reader);
        return true;
    }
    
    public void bindingOperationInputAttributes(EditableWSDLBoundOperation operation, XMLStreamReader reader) {
        for (WSDLParserExtension e : this.extensions) {
            e.bindingOperationInputAttributes(operation, reader);
        }
    }
    
    public boolean bindingOperationOutputElements(EditableWSDLBoundOperation operation, XMLStreamReader reader) {
        WSDLParserExtension[] extensions = this.extensions;
        int length = extensions.length;
        int i = 0;
        while (i < length) {
            WSDLParserExtension e = extensions[i];
            if (e.bindingOperationOutputElements(operation, reader)) {
                return true;
            }
            ++i;
        }
        XMLStreamReaderUtil.skipElement(reader);
        return true;
    }
    
    public void bindingOperationOutputAttributes(EditableWSDLBoundOperation operation, XMLStreamReader reader) {
        for (WSDLParserExtension e : this.extensions) {
            e.bindingOperationOutputAttributes(operation, reader);
        }
    }
    
    public boolean bindingOperationFaultElements(EditableWSDLBoundFault fault, XMLStreamReader reader) {
        WSDLParserExtension[] extensions = this.extensions;
        int length = extensions.length;
        int i = 0;
        while (i < length) {
            WSDLParserExtension e = extensions[i];
            if (e.bindingOperationFaultElements(fault, reader)) {
                return true;
            }
            ++i;
        }
        XMLStreamReaderUtil.skipElement(reader);
        return true;
    }
    
    public void bindingOperationFaultAttributes(EditableWSDLBoundFault fault, XMLStreamReader reader) {
        for (WSDLParserExtension e : this.extensions) {
            e.bindingOperationFaultAttributes(fault, reader);
        }
    }
    
    public void finished(WSDLParserExtensionContext context) {
        for (WSDLParserExtension e : this.extensions) {
            e.finished(context);
        }
    }
    
    public void postFinished(WSDLParserExtensionContext context) {
        for (WSDLParserExtension e : this.extensions) {
            e.postFinished(context);
        }
    }
    
    private boolean isRequiredExtension(XMLStreamReader reader) {
        String required = reader.getAttributeValue("http://schemas.xmlsoap.org/wsdl/", "required");
        return required != null && Boolean.parseBoolean(required);
    }
    
    private Locator getLocator(XMLStreamReader reader) {
        Location location = reader.getLocation();
        LocatorImpl loc = new LocatorImpl();
        loc.setSystemId(location.getSystemId());
        loc.setLineNumber(location.getLineNumber());
        return loc;
    }
}
